import { codeFrameColumns } from '@babel/code-frame';
import generate from '@babel/generator';
import { highlight } from 'cli-highlight';
import { AbnormalState, compile, setAbnormalStateReceiveListener } from '@nativescript-community/xml-ui-compiler';
import { relative } from 'path';
import { promisify } from 'util';
export default function loader(content, map) {
    const callback = this.async();
    const options = this.getOptions();
    // Error and warning handling
    setAbnormalStateReceiveListener((type, msg, range) => {
        const error = new Error(getCodeFrame(content, msg, range));
        if (type === AbnormalState.ERROR) {
            if (options.compileWithMarkupErrors) {
                this.emitError(error);
            }
            else {
                throw error;
            }
        }
        else if (type === AbnormalState.WARNING) {
            this.emitWarning(error);
        }
        else {
            throw error;
        }
    });
    loadContent(this, content, options).then((output) => {
        callback(null, output, map);
    }).catch((err) => {
        callback(err);
    });
}
function getCodeFrame(content, msg, range) {
    const location = {
        start: range[0],
        end: range[1]
    };
    const codeFrame = codeFrameColumns(content, location, {
        message: msg
    });
    return '\n' + highlight(codeFrame, {
        language: 'xml'
    });
}
async function loadContent(loader, content, options) {
    const moduleRelativePath = relative(options.appPath, loader.resourcePath);
    const { output, pathsToResolve } = compile(content, moduleRelativePath, options.platform, options.useDataBinding, options.preprocess?.attributeValueFormatter);
    if (output == null) {
        return '';
    }
    // XML parser does not work asynchronously, so we resolve requested modules once everything is done
    if (pathsToResolve?.length) {
        const resolveAsync = promisify(loader.resolve);
        await Promise.all(pathsToResolve.map(request => resolveAsync(loader.context, request)));
    }
    // Convert AST to JS
    return options.preprocess?.transformAst ? options.preprocess?.transformAst(output, generate) : generate(output).code;
}
//# sourceMappingURL=index.js.map